
#line 3 "bfs_filter_proto.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer bfs_create_buffer
#define yy_delete_buffer bfs_delete_buffer
#define yy_flex_debug bfs_flex_debug
#define yy_init_buffer bfs_init_buffer
#define yy_flush_buffer bfs_flush_buffer
#define yy_load_buffer_state bfs_load_buffer_state
#define yy_switch_to_buffer bfs_switch_to_buffer
#define yyin bfsin
#define yyleng bfsleng
#define yylex bfslex
#define yylineno bfslineno
#define yyout bfsout
#define yyrestart bfsrestart
#define yytext bfstext
#define yywrap bfswrap
#define yyalloc bfsalloc
#define yyrealloc bfsrealloc
#define yyfree bfsfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE bfsrestart(bfsin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int bfsleng;

extern FILE *bfsin, *bfsout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    /* Note: We specifically omit the test for yy_rule_can_match_eol because it requires
     *       access to the local variable yy_act. Since yyless() is a macro, it would break
     *       existing scanners that call yyless() from OUTSIDE bfslex. 
     *       One obvious solution it to make yy_act a global. I tried that, and saw
     *       a 5% performance hit in a non-bfslineno scanner, because yy_act is
     *       normally declared as a register variable-- so it is not worth it.
     */
    #define  YY_LESS_LINENO(n) \
            do { \
                int yyl;\
                for ( yyl = n; yyl < bfsleng; ++yyl )\
                    if ( bfstext[yyl] == '\n' )\
                        --bfslineno;\
            }while(0)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up bfstext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up bfstext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via bfsrestart()), so that the user can continue scanning by
	 * just pointing bfsin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when bfstext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int bfsleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow bfswrap()'s to do buffer switches
 * instead of setting up a fresh bfsin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void bfsrestart (FILE *input_file  );
void bfs_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE bfs_create_buffer (FILE *file,int size  );
void bfs_delete_buffer (YY_BUFFER_STATE b  );
void bfs_flush_buffer (YY_BUFFER_STATE b  );
void bfspush_buffer_state (YY_BUFFER_STATE new_buffer  );
void bfspop_buffer_state (void );

static void bfsensure_buffer_stack (void );
static void bfs_load_buffer_state (void );
static void bfs_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER bfs_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE bfs_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE bfs_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE bfs_scan_bytes (yyconst char *bytes,int len  );

void *bfsalloc (yy_size_t  );
void *bfsrealloc (void *,yy_size_t  );
void bfsfree (void *  );

#define yy_new_buffer bfs_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        bfsensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            bfs_create_buffer(bfsin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        bfsensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            bfs_create_buffer(bfsin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

#define bfswrap(n) 1
#define YY_SKIP_YYWRAP

typedef unsigned char YY_CHAR;

FILE *bfsin = (FILE *) 0, *bfsout = (FILE *) 0;

typedef int yy_state_type;

extern int bfslineno;

int bfslineno = 1;

extern char *bfstext;
#define yytext_ptr bfstext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up bfstext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	bfsleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 78
#define YY_END_OF_BUFFER 79
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[353] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,   79,   75,    1,   76,   75,   75,   75,    1,
        2,   77,   75,   75,   75,   75,   75,   75,   75,   75,
       75,   75,   75,   75,   16,    1,   17,   16,   16,   16,
       16,   20,    1,   21,   20,   20,   20,   20,   24,    1,
       25,   24,   24,   24,   24,   28,    1,   29,   28,   28,
       28,   28,   32,    1,   33,   32,   32,   32,   32,   36,
        1,   37,   36,   36,   36,   36,   40,    1,   41,   40,

       40,   40,   40,   44,    1,   45,   44,   44,   44,   44,
       48,    1,   49,   48,   48,   48,   48,   52,    1,   53,
       52,   52,   52,   52,   56,    1,   57,   56,   56,   56,
       56,   73,    1,   72,   73,   70,   73,   73,   60,    1,
       61,   60,   60,   60,   60,   12,    1,   13,   12,   12,
       12,   12,    1,    0,   68,   69,    0,    0,   74,    1,
        2,    2,    0,    0,    0,    0,    0,    0,    0,    0,
       62,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

        0,    0,   73,    1,   73,   68,   71,   73,   73,   73,
        0,    0,    0,    0,    0,   67,    2,    0,    4,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        5,    0,    0,   63,   64,    0,    0,   15,    0,    0,
       19,    0,    0,   23,    0,    0,   27,    0,    0,   31,
        0,    0,   35,    0,    0,   39,    0,    0,   43,    0,
        0,   47,    0,    0,   51,    0,    0,   55,    0,   73,
       67,    0,   59,    0,    0,    8,    0,    2,    0,    6,
        3,    0,   65,    0,   66,    0,    0,    0,    0,    0,
        0,    0,    0,    2,    0,    0,    0,    7,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    2,   34,    0,
        7,    9,    0,    0,    0,    0,    0,   38,    0,    0,
       26,   30,   34,    0,   42,    0,    0,    0,    0,   38,
        0,    0,   26,   30,   50,   42,   14,    0,   22,   18,
       46,    0,   50,   14,   58,   22,   18,   46,   54,   58,
       54,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    1,    4,    1,    1,    1,    1,    1,
        1,    5,    1,    1,    6,    1,    7,    8,    9,   10,
       11,   11,   11,   11,   11,   11,   11,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,   12,    1,    1,    1,    1,   13,   14,   15,   16,

       17,   18,   19,    1,   20,   21,    1,   22,   23,   24,
       25,   26,    1,   27,   28,   29,   30,   31,    1,    1,
        1,    1,   32,    1,   33,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[34] =
    {   0,
        1,    2,    1,    1,    3,    4,    1,    2,    2,    5,
        5,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[388] =
    {   0,
        0,   11,   23,    0,   55,   86,  117,  148,  179,  210,
      241,  272,  303,  334,  365,  396,  427,  458,  489,  520,
      551,  582,  613,  644,  675,  706,  737,  748,  759,  790,
      821,  852,  447, 1032,  444, 1032,  442,    1,    7,  442,
       55, 1032,   46,  421,   42,   56,   63,  416,  428,  423,
      411,  422,  420,  413, 1032,  431, 1032,  429,   72,   14,
      425, 1032,  424, 1032,  422,   73,   81,  422, 1032,  421,
     1032,  419,   87,   83,  419, 1032,  418, 1032,  416,   90,
       97,  416, 1032,  415, 1032,  413,   96,  102,  413, 1032,
      412, 1032,  410,  101,  107,  410, 1032,  409, 1032,  407,

      106,  112,  407, 1032,  405, 1032,  403,  118,  114,  403,
     1032,  402, 1032,  399,  121,  128,  399, 1032,  394, 1032,
      392,  127,  133,  392, 1032,  391, 1032,  389,  132,  138,
      389,    0,  388,    0,  139,  382,  138,  144, 1032,  386,
     1032,  384,  149,  155,  384, 1032,  383, 1032,  381,  154,
      160,  381,  380,  378, 1032, 1032,  377,  162, 1032,  377,
      158,  162,  363,  354,  345,  349,  344,  348,  357,  342,
      358,  357,  348,  339,  332,   75,  344,  332,  331,  343,
       79,  353,  100,  352,  105,  351,  120,  350,  146,  349,
      172,  348,  173,  347,  174,  346,  175,  345,  176,  343,

      182,  342,    0,  341,  182,    0, 1032,  185,  190,    0,
      187,  340,  192,  338,  336, 1032,  187,  317, 1032,  316,
      306,  325,  304,  314,  302,  301,  300,  308,  296,  295,
     1032,  305,  315,  314, 1032,  295,    0, 1032,  316,    0,
     1032,  315,    0, 1032,  314,    0, 1032,  312,    0, 1032,
      311,    0, 1032,  310,    0, 1032,  309,    0, 1032,  307,
        0, 1032,  306,    0, 1032,  301,    0, 1032,  300,  196,
        0,    0, 1032,  299,    0, 1032,  298,  218,  273, 1032,
     1032,  184, 1032,  281, 1032,  270,  269,  273,  288,  278,
      198,  274,  285,  222,  261,  263,  263,  285,  261,  260,

      277,  205,  253,  255,  255,  256,  193, 1032,  275,  245,
      269, 1032,  241,  240,  253,  238,  241,  263,  237,  250,
      260,  259,  258,  230,  256,  232,  232,  232,  228,  250,
      222,  231,  247,  245,  244,  239,  238,  210,  236,  235,
      234,  207,  221,  219,  214,  198,  125,   70,   58,    9,
        3, 1032,  884,  889,  894,  899,  904,  909,  914,  919,
      924,  929,  934,  939,  944,  949,  954,  959,  963,  967,
      970,  973,  976,  979,  982,  985,  988,  991,  994,  997,
     1001, 1006, 1010, 1013, 1017, 1022, 1026
    } ;

static yyconst flex_int16_t yy_def[388] =
    {   0,
      353,  353,  352,    3,  354,  354,  355,  355,  356,  356,
      357,  357,  358,  358,  359,  359,  360,  360,  361,  361,
      362,  362,  363,  363,  364,  364,  365,  365,  366,  366,
      367,  367,  352,  352,  352,  352,  368,  352,  352,  369,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  368,  352,  352,
      370,  352,  352,  352,  368,  352,  352,  371,  352,  352,
      352,  368,  352,  352,  372,  352,  352,  352,  368,  352,
      352,  373,  352,  352,  352,  368,  352,  352,  374,  352,
      352,  352,  368,  352,  352,  375,  352,  352,  352,  368,

      352,  352,  376,  352,  352,  352,  368,  352,  352,  377,
      352,  352,  352,  368,  352,  352,  378,  352,  352,  352,
      368,  352,  352,  379,  352,  352,  352,  368,  352,  352,
      380,  381,  381,  381,  382,  352,  381,  381,  352,  352,
      352,  368,  352,  352,  383,  352,  352,  352,  368,  352,
      352,  384,  352,  368,  352,  352,  385,  352,  352,  369,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      370,  181,  371,  183,  372,  185,  373,  187,  374,  189,
      375,  191,  376,  193,  377,  195,  378,  197,  379,  199,

      380,  201,  381,  381,  382,  381,  352,  386,  381,  381,
      383,  211,  384,  213,  385,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  181,  352,  181,  183,
      352,  183,  185,  352,  185,  187,  352,  187,  189,  352,
      189,  191,  352,  191,  193,  352,  193,  195,  352,  195,
      197,  352,  197,  199,  352,  199,  201,  352,  201,  386,
      381,  211,  352,  211,  213,  352,  213,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  387,  352,  352,

      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      387,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,    0,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352
    } ;

static yyconst flex_int16_t yy_nxt[1066] =
    {   0,
      352,   35,   36,   37,  351,  156,   38,  157,  158,  159,
      350,   39,   40,   36,   37,  158,  159,   38,   41,   41,
       41,   41,   39,   34,   35,   42,   37,   34,   34,   38,
       34,   34,   34,   34,   39,   43,   44,   34,   45,   34,
       34,   34,   46,   34,   47,   48,   34,   34,   49,   50,
       51,   52,   53,   54,   34,   34,   56,   57,   58,  351,
      163,   59,  162,  162,  162,  162,   60,  164,  166,  167,
      168,  348,  172,  169,  170,  173,  156,  156,  157,  157,
      237,  171,  158,  159,  158,  159,   61,   56,   57,   58,
      231,  156,   59,  157,  156,  232,  157,   60,  158,  159,

      156,  240,  157,  158,  159,  156,  243,  157,  158,  159,
      156,  238,  157,  158,  159,  158,  159,   61,   63,   64,
       65,  246,  156,   66,  157,  156,  347,  157,   67,  158,
      159,  156,  241,  157,  158,  159,  156,  244,  157,  158,
      159,  206,  156,  154,  208,  209,  210,  249,   68,   63,
       64,   65,  247,  156,   66,  157,  158,  159,  156,   67,
      157,  158,  159,  158,  159,  162,  162,  162,  162,  217,
      217,  217,  217,  252,  255,  258,  261,  264,  250,   68,
       70,   71,   72,  267,  206,   73,  154,  271,  272,  215,
       74,  209,  210,  275,  278,  278,  278,  278,  271,  346,

      215,  321,  322,  296,  253,  256,  259,  262,  265,  297,
       75,   70,   71,   72,  268,  350,   73,  304,  316,  273,
      344,   74,  343,  305,  276,  294,  294,  294,  294,  308,
      308,  308,  308,  317,  349,  348,  347,  346,  345,  344,
      336,   75,   77,   78,   79,  343,  334,   80,  333,  342,
      341,  330,   81,  340,  339,  338,  337,  336,  335,  323,
      334,  333,  332,  331,  330,  329,  328,  327,  326,  325,
      311,  324,   82,   77,   78,   79,  323,  320,   80,  319,
      318,  281,  315,   81,  314,  313,  311,  310,  309,  281,
      307,  306,  303,  302,  301,  300,  299,  298,  295,  277,

      274,  269,  266,   82,   84,   85,   86,  263,  260,   87,
      257,  254,  251,  248,   88,  245,  242,  239,  293,  292,
      291,  290,  281,  289,  288,  287,  286,  285,  284,  283,
      282,  281,  280,  279,   89,   84,   85,   86,  216,  277,
       87,  274,  204,  269,  266,   88,  263,  260,  257,  254,
      251,  248,  245,  242,  239,  236,  235,  234,  233,  230,
      229,  228,  227,  226,  225,   89,   91,   92,   93,  224,
      223,   94,  222,  221,  220,  219,   95,  218,  160,  216,
      155,  153,  213,  155,  153,  211,  155,  153,  207,  204,
      201,  155,  153,  199,  155,  153,   96,   91,   92,   93,

      197,  155,   94,  153,  195,  155,  153,   95,  193,  155,
      153,  191,  155,  153,  189,  155,  153,  187,  155,  153,
      185,  155,  153,  183,  155,  153,  181,   96,   98,   99,
      100,  155,  153,  101,  180,  179,  178,  177,  102,  176,
      175,  174,  165,  160,  155,  153,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  103,   98,
       99,  100,  352,  352,  101,  352,  352,  352,  352,  102,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  103,
      105,  106,  107,  352,  352,  108,  352,  352,  352,  352,

      109,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      110,  105,  106,  107,  352,  352,  108,  352,  352,  352,
      352,  109,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  110,  112,  113,  114,  352,  352,  115,  352,  352,
      352,  352,  116,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  117,  112,  113,  114,  352,  352,  115,  352,
      352,  352,  352,  116,  352,  352,  352,  352,  352,  352,

      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  117,  119,  120,  121,  352,  352,  122,
      352,  352,  352,  352,  123,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  124,  119,  120,  121,  352,  352,
      122,  352,  352,  352,  352,  123,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  124,  126,  127,  128,  352,
      352,  129,  352,  352,  352,  352,  130,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,

      352,  352,  352,  352,  352,  352,  131,  126,  127,  128,
      352,  352,  129,  352,  352,  352,  352,  130,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  131,  133,  134,
      135,  136,  352,  137,  352,  352,  352,  352,  138,  133,
      134,  135,  136,  352,  137,  352,  352,  352,  352,  138,
      140,  141,  142,  352,  352,  143,  352,  352,  352,  352,
      144,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      145,  140,  141,  142,  352,  352,  143,  352,  352,  352,

      352,  144,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  145,  147,  148,  149,  352,  352,  150,  352,  352,
      352,  352,  151,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  152,  147,  148,  149,  352,  352,  150,  352,
      352,  352,  352,  151,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  152,   34,   34,   34,   34,   34,   55,
       55,   55,   55,   55,   62,   62,   62,   62,   62,   69,

       69,   69,   69,   69,   76,   76,   76,   76,   76,   83,
       83,   83,   83,   83,   90,   90,   90,   90,   90,   97,
       97,   97,   97,   97,  104,  104,  104,  104,  104,  111,
      111,  111,  111,  111,  118,  118,  118,  118,  118,  125,
      125,  125,  125,  125,  132,  132,  132,  132,  132,  139,
      139,  139,  139,  139,  146,  146,  146,  146,  146,  154,
      154,  154,  154,  154,  161,  352,  352,  161,  182,  182,
      182,  184,  184,  184,  186,  186,  186,  188,  188,  188,
      190,  190,  190,  192,  192,  192,  194,  194,  194,  196,
      196,  196,  198,  198,  198,  200,  200,  200,  202,  202,

      202,  203,  203,  352,  203,  203,  205,  205,  205,  205,
      205,  212,  212,  212,  214,  214,  214,  215,  215,  215,
      215,  215,  270,  270,  270,  270,  270,  312,  352,  352,
      312,   33,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352
    } ;

static yyconst flex_int16_t yy_chk[1066] =
    {   0,
        0,    1,    1,    1,  351,   38,    1,   38,   39,   39,
      350,    1,    2,    2,    2,   60,   60,    2,    2,    2,
        2,    2,    2,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    5,    5,    5,  349,
       43,    5,   41,   41,   41,   41,    5,   43,   45,   45,
       46,  348,   47,   46,   46,   47,   59,   66,   59,   66,
      181,   46,   67,   67,   74,   74,    5,    6,    6,    6,
      176,   73,    6,   73,   80,  176,   80,    6,   81,   81,

       87,  183,   87,   88,   88,   94,  185,   94,   95,   95,
      101,  181,  101,  102,  102,  109,  109,    6,    7,    7,
        7,  187,  108,    7,  108,  115,  347,  115,    7,  116,
      116,  122,  183,  122,  123,  123,  129,  185,  129,  130,
      130,  135,  137,  135,  137,  138,  138,  189,    7,    8,
        8,    8,  187,  143,    8,  143,  144,  144,  150,    8,
      150,  151,  151,  158,  158,  161,  161,  161,  161,  162,
      162,  162,  162,  191,  193,  195,  197,  199,  189,    8,
        9,    9,    9,  201,  205,    9,  205,  208,  211,  208,
        9,  209,  209,  213,  217,  217,  217,  217,  270,  346,

      270,  307,  307,  282,  191,  193,  195,  197,  199,  282,
        9,   10,   10,   10,  201,  345,   10,  291,  302,  211,
      344,   10,  343,  291,  213,  278,  278,  278,  278,  294,
      294,  294,  294,  302,  342,  341,  340,  339,  338,  337,
      336,   10,   11,   11,   11,  335,  334,   11,  333,  332,
      331,  330,   11,  329,  328,  327,  326,  325,  324,  323,
      322,  321,  320,  319,  318,  317,  316,  315,  314,  313,
      311,  310,   11,   12,   12,   12,  309,  306,   12,  305,
      304,  303,  301,   12,  300,  299,  298,  297,  296,  295,
      293,  292,  290,  289,  288,  287,  286,  284,  279,  277,

      274,  269,  266,   12,   13,   13,   13,  263,  260,   13,
      257,  254,  251,  248,   13,  245,  242,  239,  236,  234,
      233,  232,  230,  229,  228,  227,  226,  225,  224,  223,
      222,  221,  220,  218,   13,   14,   14,   14,  215,  214,
       14,  212,  204,  202,  200,   14,  198,  196,  194,  192,
      190,  188,  186,  184,  182,  180,  179,  178,  177,  175,
      174,  173,  172,  171,  170,   14,   15,   15,   15,  169,
      168,   15,  167,  166,  165,  164,   15,  163,  160,  157,
      154,  153,  152,  149,  147,  145,  142,  140,  136,  133,
      131,  128,  126,  124,  121,  119,   15,   16,   16,   16,

      117,  114,   16,  112,  110,  107,  105,   16,  103,  100,
       98,   96,   93,   91,   89,   86,   84,   82,   79,   77,
       75,   72,   70,   68,   65,   63,   61,   16,   17,   17,
       17,   58,   56,   17,   54,   53,   52,   51,   17,   50,
       49,   48,   44,   40,   37,   35,   33,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,   17,   18,
       18,   18,    0,    0,   18,    0,    0,    0,    0,   18,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,   18,
       19,   19,   19,    0,    0,   19,    0,    0,    0,    0,

       19,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       19,   20,   20,   20,    0,    0,   20,    0,    0,    0,
        0,   20,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,   20,   21,   21,   21,    0,    0,   21,    0,    0,
        0,    0,   21,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,   21,   22,   22,   22,    0,    0,   22,    0,
        0,    0,    0,   22,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,   22,   23,   23,   23,    0,    0,   23,
        0,    0,    0,    0,   23,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,   23,   24,   24,   24,    0,    0,
       24,    0,    0,    0,    0,   24,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,   24,   25,   25,   25,    0,
        0,   25,    0,    0,    0,    0,   25,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,   25,   26,   26,   26,
        0,    0,   26,    0,    0,    0,    0,   26,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,   26,   27,   27,
       27,   27,    0,   27,    0,    0,    0,    0,   27,   28,
       28,   28,   28,    0,   28,    0,    0,    0,    0,   28,
       29,   29,   29,    0,    0,   29,    0,    0,    0,    0,
       29,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       29,   30,   30,   30,    0,    0,   30,    0,    0,    0,

        0,   30,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,   30,   31,   31,   31,    0,    0,   31,    0,    0,
        0,    0,   31,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,   31,   32,   32,   32,    0,    0,   32,    0,
        0,    0,    0,   32,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,   32,  353,  353,  353,  353,  353,  354,
      354,  354,  354,  354,  355,  355,  355,  355,  355,  356,

      356,  356,  356,  356,  357,  357,  357,  357,  357,  358,
      358,  358,  358,  358,  359,  359,  359,  359,  359,  360,
      360,  360,  360,  360,  361,  361,  361,  361,  361,  362,
      362,  362,  362,  362,  363,  363,  363,  363,  363,  364,
      364,  364,  364,  364,  365,  365,  365,  365,  365,  366,
      366,  366,  366,  366,  367,  367,  367,  367,  367,  368,
      368,  368,  368,  368,  369,    0,    0,  369,  370,  370,
      370,  371,  371,  371,  372,  372,  372,  373,  373,  373,
      374,  374,  374,  375,  375,  375,  376,  376,  376,  377,
      377,  377,  378,  378,  378,  379,  379,  379,  380,  380,

      380,  381,  381,    0,  381,  381,  382,  382,  382,  382,
      382,  383,  383,  383,  384,  384,  384,  385,  385,  385,
      385,  385,  386,  386,  386,  386,  386,  387,    0,    0,
      387,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  352,  352,  352
    } ;

/* Table of booleans, true if rule could match eol. */
static yyconst flex_int32_t yy_rule_can_match_eol[79] =
    {   0,
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
    0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
    0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 
    0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0,     };

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int bfs_flex_debug;
int bfs_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *bfstext;
#line 1 "bfs_filter_proto.l"
/* 
 * Copyright (c) 2003 Endace Technology Ltd, Hamilton, New Zealand.
 * All rights reserved.
 *
 * This source code is proprietary to Endace Technology Ltd and no part
 * of it may be redistributed, published or disclosed except as outlined
 * in the written contract supplied with this product.
 *
 * $Id: bfs_filter_proto.c 13619 2010-12-30 01:59:23Z karthik.sharma $ 
 * This file defines the key words which will create the high level rule 
 */ 
#define YY_NO_INPUT 1

#line 28 "bfs_filter_proto.l"
#include <stdlib.h>
#include "infiniband_proto.h"
#include "dag_config.h"

#define out stdout
/*
* Debugging the gramer defines and general output 
* IPFV2_RULE_PRINT: will output the actual recognized rules with out extra lines and comments 
* IPFV2LEX_DEBUG: Used for intial of the rule and print out of the result 
* IPFV2LEX_DEBUG_R: used for the actual rules and individual states 
* IPFV2LEX_DEBUG_COMMENT : used for comments debbuging and empty lines 
*/
#define BFS_RULE_PRINT 0
#define BFSLEX_DEBUG 0
#define BFSLEX_DEBUG_R 0
#define BFSLEX_DEBUG_COMMENT 0

/*this is done so multipl parsers to be able to be used in the same application 
each pareser just have to change this define and the prefix in the option */

#define yy_filter_rule bfs_filter_rule

bfs_filter_rule_t yy_filter_rule;
/* for the mutual exclusion between mpls and vlan filtering */
static uint8_t is_mpls_filtering = 0;
static uint8_t is_vlan_filtering = 0;
static uint32_t last_line_num = 0;
/*This is not initialised currently.may be used in future for reduced feature version of BFS.MAY BE.*/
static  bfs_capabilities_t *bfs_card_cap = NULL;

/*supporting function to be moved to some part of the library  */
#line 62 "bfs_filter_proto.l"
	
#define BINARY_TOKEN_TO_DATAMASK_64( token, bits, data,  mask ) 			\
 do {									\
	unsigned int count = 0;						\
	unsigned int index;						\
	int          buf_index;						\
	int         shift = 0;								\
    uint64_t *tmp_data = data ;               \
    uint64_t *tmp_mask = mask;                \
	if ( token == NULL ) return 0; 					\
	/* protection to work only with 1 to 64 bits */ 		\
	if ( (bits > 128) || (bits < 1) ) return 0; 			\
	if(  bits > 64 ) shift = 64;								\
	count = strlen(token);						\
									\
	/* we require this due we store everything in host order*/ 	\
	buf_index = bits - 1;						\
									\
	/* Create the entry in host byte order then convert to network byte order. */ \
	for (index = 0; index < count; index++)				\
	{								  \
                                     \
		if ((token[index] == '{') || (token[index] == '}'))	\
		{							\
			/* Do nothing. */				\
		} 							\
		else if ((token[index] == ' ') || (token[index] == '\t'))\
		{							\
			/* Do nothing. added for spaces*/		\
		}							\
		else if (token[index] == '0')				\
		{							\
			/* Data bit is already 0. */			\
			*data &= ~(1LL << (buf_index -shift));			\
									\
			/* Set mask bit to 1. */			\
			*mask |= (1LL << (buf_index - shift));			\
									\
			buf_index--;					\
		}							\
		else if (token[index] == '1')				\
		{							\
			/* Set data bit to 1. */			\
			*data |= (1LL << (buf_index -shift) );			\
			/* Set mask bit to 1. */			\
			*mask |= (1LL << (buf_index-shift));			\
									\
			buf_index--;					\
		}							\
		else if( (token[index] == '-') || (token[index] == '*'))	\
		{							\
			/* Set data bit to  0. */			\
			*data &= ~(1LL << (buf_index -shift));			\
			/* Set Mask bit to  0. */			\
			*mask &= ~(1LL << (buf_index-shift));			\
			buf_index--;					\
		}							\
		else							\
		{							\
			return 0;					\
		};							\
    if( buf_index == 63)            \
        {                               \
            data = tmp_data + 1;                     \
            mask = tmp_mask+ 1;                         \
            shift = 0;                      \
        }                       \
	};								\
									\
	/* assert(buf_index == -1);	*/				\
	/* case we provided more bits then allowed */			\
	if (buf_index != -1)						\
		return 0;						\
									\
	return 1;							\
} while(0);  								
	

#define BINARY_TOKEN_TO_DATAMASK( token, bits, data,  mask ) 			\
 do {									\
	unsigned int count = 0;						\
	unsigned int index;						\
	int          buf_index;						\
									\
	if ( token == NULL ) return 0; 					\
	/* protection to work only with 1 to 64 bits */ 		\
	if ( (bits > 64) || (bits < 1) ) return 0; 			\
									\
	count = strlen(token);						\
									\
	/* we require this due we store everything in host order*/ 	\
	buf_index = bits - 1;						\
									\
	/* Create the entry in host byte order then convert to network byte order. */ \
	for (index = 0; index < count; index++)				\
	{								\
		if ((token[index] == '{') || (token[index] == '}'))	\
		{							\
			/* Do nothing. */				\
		} 							\
		else if ((token[index] == ' ') || (token[index] == '\t'))\
		{							\
			/* Do nothing. added for spaces*/		\
		}							\
		else if (token[index] == '0')				\
		{							\
			/* Data bit is already 0. */			\
			*data &= ~(1 << buf_index);			\
									\
			/* Set mask bit to 1. */			\
			*mask |= (1 << buf_index);			\
									\
			buf_index--;					\
		}							\
		else if (token[index] == '1')				\
		{							\
			/* Set data bit to 1. */			\
			*data |= (1 << buf_index);			\
			/* Set mask bit to 1. */			\
			*mask |= (1 << buf_index);			\
									\
			buf_index--;					\
		}							\
		else if( (token[index] == '-') || (token[index] == '*'))	\
		{							\
			/* Set data bit to  0. */			\
			*data &= ~(1 << buf_index);			\
			/* Set Mask bit to  0. */			\
			*mask &= ~(1 << buf_index);			\
			buf_index--;					\
		}							\
		else							\
		{							\
			return 0;					\
		};							\
	};								\
									\
	/* assert(buf_index == -1);	*/				\
	/* case we provided more bits then allowed */			\
	if (buf_index != -1)						\
		return 0;						\
									\
	return 1;							\
} while(0);  			
//TODO: FIXME:
#define HEX_TOKEN_TO_DATAMASK( token, bits, data,  mask ) 		\
 do {									\
	unsigned int count = 0;						\
	unsigned int index;						\
	int          buf_index;						\
	uint64_t nible_mask;						\
									\
	if ( token == NULL ) return 0; 					\
	/* protection to work only with 1 to 64 bits */ 		\
	if ( (bits > 64) || (bits < 1) ) return 0; 			\
									\
	count = strlen(token);						\
	/* protection from empty strings */				\
	if ( count < 1 ) return 0; 					\
									\
	/* we require this due we store everything in host order*/ 	\
	buf_index = 0;							\
	/* Create the entry in host byte order then  we invert the proccecing. */ \
	for (index = count-1; index >= 0; index--)				\
	{								\
	/* calculates the nible mask in case we have 11 bits 2 nibles(16) */	\
		nible_mask = 0xF; 					\
		if( (bits - buf_index) < 4 ) {				\
			nible_mask = 0xF >>  (bits - buf_index);	\
		}; 							\
									\
		if ((token[index] == '{') || (token[index] == '}'))	\
		{							\
			/* Do nothing. */				\
		} 							\
		else if( (token[index] == ' ') || (token[index] == '\t') || (token[index] == 0))	\
		{							\
			/* Do nothing. added for spaces*/		\
		}							\
		else if ((token[index] == 'x') || (token[index] == '\t'))\
		{							\
			/* we expect 0x000 procesing backwards 		\
			 and as soon as we get to x we are finished */	\
			return 1;					\
		}							\
		else if( (token[index] >= '0')  && (token[index] < '9') )	\
		{							\
			/* Set data bits. */				\
			*data |= ((token[index]-'0') << buf_index)&nible_mask;	\
			*data &= ~(((token[index]-'0') << buf_index)&nible_mask);	\
			/* Set mask 4 bits to 0xf. */			\
			*mask |= (0xF << buf_index);			\
			buf_index+=4;					\
		}							\
		else if( (token[index] >= 'a') || (token[index] <= 'f'))	\
		{							\
			/* Set data bits. */				\
			*data |= ((token[index]-'a') << buf_index)&nible_mask;	\
			*data &= ~(((token[index]-'a') << buf_index)&nible_mask);	\
			/* Set mask 4 bits to 0xf. */			\
			*mask |= (nible_mask << buf_index);		\
									\
			buf_index+=4;					\
		}							\
		else if (token[index] == '-'|| token[index] == '*')	\
		{							\
			/* Address bit is already 0. */			\
			/* Mask bit is already 0. */			\
			buf_index+=4;					\
		}							\
		else							\
		{							\
			return 0;					\
		};							\
	};								\
									\
	/* case we provided more or less nibles then allowed */		\
	if ((buf_index - bits) > 0 )					\
		return 0;						\
	if( (bits - buf_index) > 3 )					\
		return 0;						\
									\
	return 1;							\
} while(0);  								

		
/**
 * Internal Functions to parse a bit mask in format "-1-0-" and to return mask and data value works with up to 64 bit data and mask 
 * the matching will ignore the following symbols '{' , space , tab, '}'
 *     ------------------------------------------
 * Where '-' can be replaced with either '0' or '1'.
 *
 * @param[in]  token    Pointer to the string token being parsed.
 * @param[in]  bits     how many bits are expected and how many bits will be returned in the result 
 * @param[in]  data     pinter to 64 bit data which will be returned 
 * @param[in]  mask     pinter to 64 bit data which will be returned 
 *
 * @returns data         is the data in host order 
 * @returns mask         is the mask in host order 
 * @returns              Returns true if the token contained a valid value, otherwise
 *                       false is returned.
 *
 * @sa 
 */

static int bit_entry_8(const char * token, int bits, uint8_t * data, uint8_t * mask )
{

	BINARY_TOKEN_TO_DATAMASK( token, bits, data, mask );
};

static int bit_entry_16(const char * token, int bits, uint16_t * data, uint16_t * mask )
{

	BINARY_TOKEN_TO_DATAMASK( token, bits, data, mask );
};

static  int bit_entry_32(const char * token, int bits, uint32_t * data, uint32_t * mask )
{

	BINARY_TOKEN_TO_DATAMASK( token, bits, data, mask);
}
#if 0
static  int bit_entry_64(const char * token, int bits, uint64_t * data, uint64_t * mask )
{

	BINARY_TOKEN_TO_DATAMASK_64( token, bits, data, mask);
}
	 
static int hex_entry_16(const char * token, int bits, uint16_t * data, uint16_t * mask )
{

	HEX_TOKEN_TO_DATAMASK( token, bits, data, mask );
};
#endif

static  int bit_entry_128(const char * token, int bits, uint64_t * data, uint64_t * mask )
{

	BINARY_TOKEN_TO_DATAMASK_64( token, bits, data, mask);
}
static int parse_ip_addr(const char * token, uint64_t * data, uint64_t * mask)
{
    
    if ( (kTypeUnknown == yy_filter_rule.ip_address_type) || (k32Bit == yy_filter_rule.ip_address_type) )
    {
        if (bit_entry_32 ( token, 32, (uint32_t*) data, (uint32_t*) mask) )
        {
            yy_filter_rule.ip_address_type = k32Bit;
            return 1;
        }
    }
    memset( data, 0, sizeof(uint64_t) ) ;
    memset ( mask, 0, sizeof ( uint64_t) );
    if ( (kTypeUnknown == yy_filter_rule.ip_address_type) || (k128Bit == yy_filter_rule.ip_address_type) )
    {
        if ( bit_entry_128(token, 128, data, mask) )
        {
            /*its a 128 bit ip address */
            yy_filter_rule.ip_address_type = k128Bit;
            return 1;
        }
    }
    /* some error - its not 32 bit or 128 bit */
    return 0;
}
#line 1199 "bfs_filter_proto.c"

#define INITIAL 0
#define S_bfs_rule 1
#define S_l2_proto 2
#define S_mpls_top 3
#define S_mpls_bottom 4
#define S_vlan_1 5
#define S_vlan_2 6
#define S_dst_ip 7
#define S_src_ip 8
#define S_ip_prot 9
#define S_src_port 10
#define S_dst_port 11
#define S_tcp_flags 12
#define S_comment 13
#define S_label_cnt 14
#define S_iface 15

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int bfslex_destroy (void );

int bfsget_debug (void );

void bfsset_debug (int debug_flag  );

YY_EXTRA_TYPE bfsget_extra (void );

void bfsset_extra (YY_EXTRA_TYPE user_defined  );

FILE *bfsget_in (void );

void bfsset_in  (FILE * in_str  );

FILE *bfsget_out (void );

void bfsset_out  (FILE * out_str  );

int bfsget_leng (void );

char *bfsget_text (void );

int bfsget_lineno (void );

void bfsset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int bfswrap (void );
#else
extern int bfswrap (void );
#endif
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

        static int yy_start_stack_ptr = 0;
        static int yy_start_stack_depth = 0;
        static int *yy_start_stack = NULL;
    
    static void yy_push_state (int new_state );
    
    static void yy_pop_state (void );
    
    static int yy_top_state (void );
    
/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( bfstext, bfsleng, 1, bfsout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		unsigned n; \
		for ( n = 0; n < max_size && \
			     (c = getc( bfsin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( bfsin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, bfsin))==0 && ferror(bfsin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(bfsin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int bfslex (void);

#define YY_DECL int bfslex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after bfstext and bfsleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( bfsleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(bfstext[bfsleng - 1] == '\n'); \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
#line 370 "bfs_filter_proto.l"

	/* ignore spaces at any level */		
#line 1410 "bfs_filter_proto.c"

	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! bfsin )
			bfsin = stdin;

		if ( ! bfsout )
			bfsout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			bfsensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				bfs_create_buffer(bfsin,YY_BUF_SIZE );
		}

		bfs_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of bfstext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 353 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 1032 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

		if ( yy_act != YY_END_OF_BUFFER && yy_rule_can_match_eol[yy_act] )
			{
			int yyl;
			for ( yyl = 0; yyl < bfsleng; ++yyl )
				if ( bfstext[yyl] == '\n' )
					   
    bfslineno++;
;
			}

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 372 "bfs_filter_proto.l"
;
	YY_BREAK
/* Start of rule and a TAG recognition */
case 2:
YY_RULE_SETUP
#line 376 "bfs_filter_proto.l"
{
			//clear the current rule to all zero
			memset(&yy_filter_rule,0,sizeof(yy_filter_rule));
	 		yy_filter_rule.user_tag = atoi(bfstext);
			BEGIN(S_bfs_rule);
	#if BFSLEX_DEBUG
	 		fprintf(out,"START RULE: user tag: %d line_no: %d",yy_filter_rule.user_tag,bfslineno );
	 		fprintf(out, "\n");
	#endif 
		}
	YY_BREAK
/* Action for the filter */		
case 3:
YY_RULE_SETUP
#line 387 "bfs_filter_proto.l"
{
			yy_filter_rule.action = ( (bfstext[0] == 'a') || (bfstext[0] == 'p') )?0:1;
			yy_filter_rule.user_class = 0;
			#if BFS_RULE_PRINT
			fprintf( out,"ACTION: %d line_no: %d\n",yy_filter_rule.action, bfslineno );
			#endif 
		};
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 394 "bfs_filter_proto.l"
{
            int tmp_action  = yy_filter_rule.action ;
            uint16_t tmp_user_tag = yy_filter_rule.user_tag;
            uint8_t tmp_user_class = yy_filter_rule.user_class;
			memset(&yy_filter_rule, 0 , sizeof(bfs_filter_rule_t));
			#if BFS_RULE_PRINT
			fprintf( out,"Steer: %d line_no: %d\n",yy_filter_rule.user_class , bfslineno );
			#endif
            yy_filter_rule.action = tmp_action;
            yy_filter_rule.user_tag = tmp_user_tag;
            yy_filter_rule.user_class = tmp_user_class;
		};
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 407 "bfs_filter_proto.l"
{
			yy_filter_rule.user_class |= 0x01;
			#if BFS_RULE_PRINT
			fprintf( out,"Steer: %d line_no: %d\n",yy_filter_rule.user_class , bfslineno );
			#endif
		};
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 413 "bfs_filter_proto.l"
{
			yy_filter_rule.user_class |= 0x02;
			#if BFS_RULE_PRINT
			fprintf( out,"Steer: %d line_no: %d\n",yy_filter_rule.user_class , bfslineno );
			#endif
		};
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 420 "bfs_filter_proto.l"
{ 	
			BEGIN(S_iface); 
		}
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 423 "bfs_filter_proto.l"
{ 
		
		if (! bit_entry_8 ( bfstext, 2 , 
			&yy_filter_rule.iface.data, 
			&yy_filter_rule.iface.mask) )
		{		
			fprintf( out, "ERROR: Bad iface parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "iface 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.iface.data, yy_filter_rule.iface.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 437 "bfs_filter_proto.l"
{ 	
		yy_filter_rule.iface.data  = atoi(&bfstext[5]);
                yy_filter_rule.iface.mask  = 0x3;
            #if BFS_RULE_PRINT
			fprintf( out, "iface 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.iface.data, yy_filter_rule.iface.mask, bfslineno );
	    #endif
		}
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 445 "bfs_filter_proto.l"
{ 	
			BEGIN(S_iface); 
		}
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 448 "bfs_filter_proto.l"
{ 
		
		if (! bit_entry_8 ( bfstext, 2 , 
			&yy_filter_rule.iface.data, 
			&yy_filter_rule.iface.mask) )
		{		
			fprintf( out, "ERROR: Bad iface parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "iface 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.iface.data, yy_filter_rule.iface.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 462 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after iface  '{---}' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 13:
/* rule 13 can match eol */
YY_RULE_SETUP
#line 466 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
		}
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 471 "bfs_filter_proto.l"
{ 	
			BEGIN(S_l2_proto); 
		}
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 475 "bfs_filter_proto.l"
{ 
		if (! bit_entry_16 ( bfstext, 16 , 
			&yy_filter_rule.l2_proto.data, 
			&yy_filter_rule.l2_proto.mask) )
		{		
			fprintf( out, "ERROR: Bad l2-proto parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "l2-proto 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.l2_proto.data, yy_filter_rule.l2_proto.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 488 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after l2-proto  '{---- ---- ---- ---- }' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 17:
/* rule 17 can match eol */
YY_RULE_SETUP
#line 492 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 498 "bfs_filter_proto.l"
{ 	
			BEGIN(S_mpls_top); 
		}
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 502 "bfs_filter_proto.l"
{ 
        if ( (bfs_card_cap) && !bfs_card_cap->mpls_filtering_supported )
        {
            fprintf( out, "Error: mpls filtering is not supported by the F/W. line_no: %d\n",bfslineno );
			return T_ERROR;
        };
        /* check for vlan filtering in the ruleset */
		if ( is_vlan_filtering )
        {
            fprintf( out, "ERROR: MPLS filtering not allowed in line  %d. Previous declaration of VLAN filtering in line %d\n",bfslineno, last_line_num );
			return T_ERROR;
        };
    
        if (! bit_entry_32 ( bfstext, 32 , 
			&yy_filter_rule.mpls_top.data, 
			&yy_filter_rule.mpls_top.mask) )
		{		
			fprintf( out, "ERROR: Bad mpls-top parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
        is_mpls_filtering = 1;
        last_line_num = bfslineno;
		#if BFS_RULE_PRINT
		fprintf( out, "mpls-top 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.mpls_top.data, yy_filter_rule.mpls_top.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 529 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after mpls-top  '{---- ---- ---- ---- ---- ---- ---- ----}' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 21:
/* rule 21 can match eol */
YY_RULE_SETUP
#line 533 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 538 "bfs_filter_proto.l"
{ 	
			BEGIN(S_mpls_bottom); 
		}
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 541 "bfs_filter_proto.l"
{ 
        
        if ((bfs_card_cap) &&  !bfs_card_cap->mpls_filtering_supported )
        {
            fprintf( out, "Error: mpls filtering is not supported by the F/W. line_no: %d\n",bfslineno );
			return T_ERROR;
        };
		 /* check for vlan filtering in the ruleset */
		if ( is_vlan_filtering )
        {
            fprintf( out, "ERROR: MPLS filtering not allowed in line  %d. Previous declaration of VLAN filtering in line %d\n",bfslineno, last_line_num );
			return T_ERROR;
        };
		if (! bit_entry_32 ( bfstext, 32 , 
			&yy_filter_rule.mpls_bottom.data, 
			&yy_filter_rule.mpls_bottom.mask) )
		{		
			fprintf( out, "ERROR: Bad mpls-btm parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
         
        is_mpls_filtering = 1;
        last_line_num = bfslineno;
		#if BFS_RULE_PRINT
		fprintf( out, "mpls-btm 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.mpls_bottom.data, yy_filter_rule.mpls_bottom.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 569 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after mpls-btm  '{---- ---- ---- ---- ---- ---- ---- ----}' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 25:
/* rule 25 can match eol */
YY_RULE_SETUP
#line 573 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
/* vlan-1 */
case 26:
YY_RULE_SETUP
#line 578 "bfs_filter_proto.l"
{ 	
			BEGIN(S_vlan_1); 
		}
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 581 "bfs_filter_proto.l"
{ 
        if ( (bfs_card_cap) && !bfs_card_cap->vlan_filtering_supported )
        {
            fprintf( out, "Error: VLAN filtering is not supported by the F/W. line_no: %d\n",bfslineno );
			return T_ERROR;
        };
		/* check for mpls filtering in the ruleset */
		if ( is_mpls_filtering )
        {
            fprintf( out, "ERROR: VLAN filtering not allowed in line  %d. Previous declaration of MPLS filtering in line %d\n",bfslineno, last_line_num );
			return T_ERROR;
        };
		
		if (! bit_entry_16 ( bfstext, 16 , 
			&yy_filter_rule.vlan_1.data, 
			&yy_filter_rule.vlan_1.mask) )
		{		
			fprintf( out, "ERROR: Bad vlan-1 parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
        
        is_vlan_filtering = 1;
        last_line_num = bfslineno;
		#if BFS_RULE_PRINT
		fprintf( out, "vlan-1 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.vlan_1.data, yy_filter_rule.vlan_1.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 609 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after vlan-1  '{---- ---- ---- ---- }' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 29:
/* rule 29 can match eol */
YY_RULE_SETUP
#line 613 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
/* vlan-2 */
case 30:
YY_RULE_SETUP
#line 620 "bfs_filter_proto.l"
{ 	
			BEGIN(S_vlan_2); 
		}
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 623 "bfs_filter_proto.l"
{ 
        /* check for mpls filtering in the ruleset */
		if ( (bfs_card_cap) && !bfs_card_cap->vlan_filtering_supported )
        {
            fprintf( out, "Error: VLAN filtering is not supported by the F/W. line_no: %d\n",bfslineno );
			return T_ERROR;
        };
        if ( is_mpls_filtering )
        {
            fprintf( out, "ERROR: VLAN filtering not allowed in line  %d. Previous declaration of MPLS filtering in line %d\n",bfslineno, last_line_num );
			return T_ERROR;
        };
		if (! bit_entry_16 ( bfstext, 16 , 
			&yy_filter_rule.vlan_2.data, 
			&yy_filter_rule.vlan_2.mask) )
		{		
			fprintf( out, "ERROR: Bad vlan-2 parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
        
        is_vlan_filtering = 1;
        last_line_num = bfslineno;
		#if BFS_RULE_PRINT
		fprintf( out, "vlan-2 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.vlan_2.data, yy_filter_rule.vlan_2.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 650 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after vlan-2  '{---- ---- ---- ----}' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 33:
/* rule 33 can match eol */
YY_RULE_SETUP
#line 654 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
/* dst-ip */
case 34:
YY_RULE_SETUP
#line 661 "bfs_filter_proto.l"
{ 	
			BEGIN(S_dst_ip); 
		}
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 664 "bfs_filter_proto.l"
{ 

		if (! parse_ip_addr ( bfstext, yy_filter_rule.dst_ip.data, 
			yy_filter_rule.dst_ip.mask)) 
		{		
			fprintf( out, "ERROR: Bad dst-ip parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
        if ( k128Bit == yy_filter_rule.ip_address_type)
        {
            if ((bfs_card_cap) &&  ((!bfs_card_cap->ipv6_supported) || ( !bfs_card_cap->width576_supported)) )
            {
                fprintf( out, "ERROR: Firmware does not support 128 bit dst-ip parameter line_no: %d\n",bfslineno );
                return T_ERROR;
            }
        };
		#if BFS_RULE_PRINT
        if( k128Bit != yy_filter_rule.ip_address_type )
    		  fprintf( out, "dst-ip 0x%04x,0x%04x line_no: %d\n",(uint32_t)yy_filter_rule.dst_ip.data[0], (uint32_t)yy_filter_rule.dst_ip.mask[0], bfslineno );
        else
            fprintf( out, "dst-ip128 0x%016"PRIx64"%016"PRIx64",0x%016"PRIx64"%016"PRIx64" line_no: %d\n",yy_filter_rule.dst_ip.data[0],yy_filter_rule.dst_ip.data[1], yy_filter_rule.dst_ip.mask[0],yy_filter_rule.dst_ip.mask[1], bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 688 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after dst-ip  '{---- ---- ---- ---- ---- ---- ---- ----}' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 37:
/* rule 37 can match eol */
YY_RULE_SETUP
#line 692 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
/* src-ip */
case 38:
YY_RULE_SETUP
#line 699 "bfs_filter_proto.l"
{ 	
			BEGIN(S_src_ip); 
		}
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 702 "bfs_filter_proto.l"
{ 
		
		if (! parse_ip_addr ( bfstext, yy_filter_rule.src_ip.data, 
			yy_filter_rule.src_ip.mask) )
		{		
			fprintf( out, "ERROR: Bad src-ip parameter line_no: %d\n",bfslineno );
            fprintf( out, "Make sure that you have not mixed 32 bit and 128bit addresses\n");
			return T_ERROR;
		};
        if ( k128Bit == yy_filter_rule.ip_address_type)
        {
            if ((bfs_card_cap) && ( (!bfs_card_cap->ipv6_supported) || ( !bfs_card_cap->width576_supported) ) )
            {
                fprintf( out, "ERROR: Firmware does not support 128 bit src-ip parameter line_no: %d\n",bfslineno );
                return T_ERROR;
            }
        };
		#if BFS_RULE_PRINT
		if( k128Bit != yy_filter_rule.ip_address_type )
    		  fprintf( out, "src-ip 0x%04x,0x%04x line_no: %d\n",(uint32_t)yy_filter_rule.src_ip.data[0], (uint32_t)yy_filter_rule.src_ip.mask[0], bfslineno );
        else
            fprintf( out, "src-ip 0x%016"PRIx64"%016"PRIx64",0x%016"PRIx64"%016"PRIx64" line_no: %d\n",yy_filter_rule.src_ip.data[0],yy_filter_rule.src_ip.data[1], yy_filter_rule.src_ip.mask[0],yy_filter_rule.src_ip.mask[1], bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 727 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after src-ip  '{---- ---- ---- ---- ---- ---- ---- ----}' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 41:
/* rule 41 can match eol */
YY_RULE_SETUP
#line 731 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
/* ip-prot */
case 42:
YY_RULE_SETUP
#line 738 "bfs_filter_proto.l"
{ 	
			BEGIN(S_ip_prot); 
		}
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 741 "bfs_filter_proto.l"
{ 
		
		if (! bit_entry_8 ( bfstext, 8 , 
			&yy_filter_rule.ip_prot.data, 
			&yy_filter_rule.ip_prot.mask) )
		{		
			fprintf( out, "ERROR: Bad ip-prot parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "ip-prot 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.ip_prot.data, yy_filter_rule.ip_prot.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 755 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after ip-prot  '{---- ---- }' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 45:
/* rule 45 can match eol */
YY_RULE_SETUP
#line 759 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
/* src-port */
case 46:
YY_RULE_SETUP
#line 766 "bfs_filter_proto.l"
{ 	
			BEGIN(S_src_port); 
		}
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 769 "bfs_filter_proto.l"
{ 
		
		if (! bit_entry_16 ( bfstext, 16 , 
			&yy_filter_rule.src_port.data, 
			&yy_filter_rule.src_port.mask) )
		{		
			fprintf( out, "ERROR: Bad src-port parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "src-port 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.src_port.data, yy_filter_rule.src_port.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 783 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after src-port  '{---- ---- ---- ---- }' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 49:
/* rule 49 can match eol */
YY_RULE_SETUP
#line 787 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
/* dst-port */
case 50:
YY_RULE_SETUP
#line 794 "bfs_filter_proto.l"
{ 	
			BEGIN(S_dst_port); 
		}
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 797 "bfs_filter_proto.l"
{ 
		
		if (! bit_entry_16 ( bfstext, 16 , 
			&yy_filter_rule.dst_port.data, 
			&yy_filter_rule.dst_port.mask) )
		{		
			fprintf( out, "ERROR: Bad dst-port parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "dst-port 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.dst_port.data, yy_filter_rule.dst_port.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 811 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after dst-port  '{---- ---- ---- ---- }' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 53:
/* rule 53 can match eol */
YY_RULE_SETUP
#line 815 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}
	YY_BREAK
/* tcp-flags */
case 54:
YY_RULE_SETUP
#line 822 "bfs_filter_proto.l"
{ 	
			BEGIN(S_tcp_flags); 
		}
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 825 "bfs_filter_proto.l"
{ 
		
		if (! bit_entry_8 ( bfstext, 8 , 
			&yy_filter_rule.tcp_flags.data, 
			&yy_filter_rule.tcp_flags.mask) )
		{		
			fprintf( out, "ERROR: Bad tcp-flags parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "tcp-flags 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.tcp_flags.data, yy_filter_rule.tcp_flags.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 839 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after tcp-flags  '{---- ---- }' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 57:
/* rule 57 can match eol */
YY_RULE_SETUP
#line 843 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 848 "bfs_filter_proto.l"
{ 	
			BEGIN(S_label_cnt); 
		}
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 851 "bfs_filter_proto.l"
{ 
		if (! bit_entry_8 ( bfstext, 3 , 
			&yy_filter_rule.label_cnt.data, 
			&yy_filter_rule.label_cnt.mask) )
		{		
			fprintf( out, "ERROR: Bad label-cnt parameter line_no: %d\n",bfslineno );
			return T_ERROR;
		};
		#if BFS_RULE_PRINT
		fprintf( out, "label-cnt 0x%04x,0x%04x line_no: %d\n",yy_filter_rule.label_cnt.data, yy_filter_rule.label_cnt.mask, bfslineno );
		#endif 
		BEGIN(S_bfs_rule);
	}
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 864 "bfs_filter_proto.l"
{
			fprintf( out,"ERROR: wrong format starting with %s on line: %d after label-cnt  '{---}' expected! \n",bfstext, bfslineno);
			return T_ERROR;
	}	
	YY_BREAK
case 61:
/* rule 61 can match eol */
YY_RULE_SETUP
#line 868 "bfs_filter_proto.l"
{
			fprintf(out,"ERROR: unexpected enf of line on line: %d ! \n", bfslineno-1);
			return T_ERROR;
	}	
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 873 "bfs_filter_proto.l"
{ 	
				yy_filter_rule.ip_prot.data = 0x00;
                yy_filter_rule.ip_prot.mask = 0x00;
            #if BFS_RULE_PRINT
			fprintf( out,"ip ( ip_prot = 0x00) line_no: %d\n", bfslineno );
			#endif
		}	
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 881 "bfs_filter_proto.l"
{ 	
				yy_filter_rule.ip_prot.data = 0x06;
                yy_filter_rule.ip_prot.mask = 0xff;
            #if BFS_RULE_PRINT
			fprintf( out,"tcp ( ip_prot = 0x06) line_no: %d\n", bfslineno );
			#endif
		}	
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 889 "bfs_filter_proto.l"
{ 	
				yy_filter_rule.ip_prot.data = 0x11;
                yy_filter_rule.ip_prot.mask = 0xff;
            #if BFS_RULE_PRINT
			fprintf( out,"udp ( ip_prot = 0x11) line_no: %d\n", bfslineno );
			#endif
		}	
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 896 "bfs_filter_proto.l"
{ 	
				yy_filter_rule.ip_prot.data = 0x01;
                yy_filter_rule.ip_prot.mask = 0xff;
            #if BFS_RULE_PRINT
			fprintf( out,"icmp ( ip_prot = 0x01) line_no: %d\n", bfslineno );
			#endif
		}	
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 903 "bfs_filter_proto.l"
{ 	
				yy_filter_rule.ip_prot.data = 0x02;
                yy_filter_rule.ip_prot.mask = 0xff;
            #if BFS_RULE_PRINT
			fprintf( out,"igmp ( ip_prot = 0x02) line_no: %d\n", bfslineno );
			#endif
		}	
	YY_BREAK
/* Implementation for adding comments in the rules 
	 *  single line commnet // to the end of the line 
	 * multiline comment from / * to the * / at any place multiple lines 
	 * and the status is at the point of begining of comment
	 */
case 67:
/* rule 67 can match eol */
YY_RULE_SETUP
#line 919 "bfs_filter_proto.l"
{	
	#if BFSLEX_DEBUG_COMMENT
			fprintf( out, "Single line Comment lineno:%d ", bfslineno-1); 
			ECHO;
			fprintf( out,"\n");
	#endif
		}	
	YY_BREAK
case 68:
/* rule 68 can match eol */
YY_RULE_SETUP
#line 926 "bfs_filter_proto.l"
{	
	#if BFSLEX_DEBUG_COMMENT
			fprintf( out, "Single line Comment lineno:%d ", bfslineno-1); 
			ECHO;
			fprintf( out,"\n");
	#endif
		}	
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 933 "bfs_filter_proto.l"
{
			yy_push_state(S_comment);
	#if BFSLEX_DEBUG_COMMENT
			fprintf( out, "SATRT Multy line Commentlineno:%d\n", bfslineno); 
	#endif 
		}
	YY_BREAK
/* warning there is a rule which may overight this one <*>.  */				
case 70:
YY_RULE_SETUP
#line 940 "bfs_filter_proto.l"
; 
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 941 "bfs_filter_proto.l"
{
			yy_pop_state();
	#if BFSLEX_DEBUG_COMMENT
			fprintf( out, "FINISH Multy line Commentlineno:%d\n", bfslineno); 
	#endif
  
		}
	YY_BREAK
case 72:
/* rule 72 can match eol */
YY_RULE_SETUP
#line 948 "bfs_filter_proto.l"
; 
	YY_BREAK
case 73:
/* rule 73 can match eol */
YY_RULE_SETUP
#line 949 "bfs_filter_proto.l"
; 
	YY_BREAK
/* Common things like:
	* A. concatenate multiple lines for one rule \ \n 
	* B. unrecognized charecters any 
	* C. new line and new rule \n,
	*/
case 74:
/* rule 74 can match eol */
YY_RULE_SETUP
#line 957 "bfs_filter_proto.l"
{
			#if BFSLEX_DEBUG_COMMENT
			fprintf( out, "RULE continue to a new lineno:%d\n", bfslineno ); 
			#endif 
			}
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 962 "bfs_filter_proto.l"
{
			fprintf( out, "\nUnrecognized character: %s lineno:%d\n", bfstext, bfslineno ); 
			return T_ERROR;
	}
	YY_BREAK
case 76:
/* rule 76 can match eol */
YY_RULE_SETUP
#line 967 "bfs_filter_proto.l"
{
	#if BFSLEX_DEBUG_COMMENT
		fprintf( out,"Empty line just skip:%d\n",bfslineno-1);
	#endif 
	}
	YY_BREAK
case 77:
/* rule 77 can match eol */
YY_RULE_SETUP
#line 973 "bfs_filter_proto.l"
{
	#if BFSLEX_DEBUG
		fprintf( out, "User tag: %d, action: %s, steering: %d \n", 
   	    		yy_filter_rule.user_tag, (yy_filter_rule.action == 0)?"through":"drop" , yy_filter_rule.user_class );
        fprintf( out, "iface 0x%04x,0x%04x \n",yy_filter_rule.iface.data, yy_filter_rule.iface.mask);
        fprintf( out, "l2-proto 0x%04x,0x%04x\n",yy_filter_rule.l2_proto.data, yy_filter_rule.l2_proto.mask);
        fprintf( out, "mpls-top 0x%04x,0x%04x\n",yy_filter_rule.mpls_top.data, yy_filter_rule.mpls_top.mask);
        fprintf( out, "mpls-btm 0x%04x,0x%04x\n",yy_filter_rule.mpls_bottom.data, yy_filter_rule.mpls_bottom.mask);
        fprintf( out, "vlan-1 0x%04x,0x%04x\n",yy_filter_rule.vlan_1.data, yy_filter_rule.vlan_1.mask);
        fprintf( out, "vlan-2 0x%04x,0x%04x\n",yy_filter_rule.vlan_2.data, yy_filter_rule.vlan_2.mask);
        if ( k128Bit != yy_filter_rule.ip_address_type )
        {
            fprintf( out, "dst-ip 0x%04x,0x%04x\n",(uint32_t)yy_filter_rule.dst_ip.data[0], (uint32_t)yy_filter_rule.dst_ip.mask[0]);
            fprintf( out, "src-ip 0x%04x,0x%04x\n",(uint32_t) yy_filter_rule.src_ip.data[0], (uint32_t) yy_filter_rule.src_ip.mask[0]);
        }
        else 
        {
            fprintf( out, "dst-ip128 0x%016"PRIx64"%016"PRIx64",0x%016"PRIx64"%016"PRIx64" line_no: %d\n",yy_filter_rule.dst_ip.data[0],yy_filter_rule.dst_ip.data[1], yy_filter_rule.dst_ip.mask[0],yy_filter_rule.dst_ip.mask[1], bfslineno );
            fprintf( out, "src-ip 0x%016"PRIx64"%016"PRIx64",0x%016"PRIx64"%016"PRIx64" line_no: %d\n",yy_filter_rule.src_ip.data[0],yy_filter_rule.src_ip.data[1], yy_filter_rule.src_ip.mask[0],yy_filter_rule.src_ip.mask[1], bfslineno );
        }
        fprintf( out, "ip-prot 0x%04x,0x%04x\n",yy_filter_rule.ip_prot.data, yy_filter_rule.ip_prot.mask);
        fprintf( out, "src-port 0x%04x,0x%04x\n",yy_filter_rule.src_port.data, yy_filter_rule.src_port.mask);
        fprintf( out, "dst-port 0x%04x,0x%04x\n",yy_filter_rule.dst_port.data, yy_filter_rule.dst_port.mask);
        fprintf( out, "tcp-flags 0x%04x,0x%04x\n",yy_filter_rule.tcp_flags.data, yy_filter_rule.tcp_flags.mask);
		fprintf( out,"END of rule: on lineno:%d\n",bfslineno-1);
	#endif
		BEGIN(INITIAL);
		/* 
		 * This can be used to change the default behavior 
		 * if you disable the return will retutn at the end of the file or error 
		 * usfull for debugging to be set to 0 or in case we change the sacnner to return set instead of a single rule
		 */
		#if 1
		 return T_RULE_DONE;
		#endif 
	}
	YY_BREAK
case YY_STATE_EOF(S_bfs_rule):
#line 1009 "bfs_filter_proto.l"
{
	#if BFSLEX_DEBUG
		fprintf( out, "User tag: %d, action: %s, steering: %d \n", 
   	    		yy_filter_rule.user_tag, (yy_filter_rule.action == 0)?"through":"drop" , yy_filter_rule.user_class );
        fprintf( out, "iface 0x%04x,0x%04x \n",yy_filter_rule.iface.data, yy_filter_rule.iface.mask);
        fprintf( out, "l2-proto 0x%04x,0x%04x\n",yy_filter_rule.l2_proto.data, yy_filter_rule.l2_proto.mask);
        fprintf( out, "mpls-top 0x%04x,0x%04x\n",yy_filter_rule.mpls_top.data, yy_filter_rule.mpls_top.mask);
        fprintf( out, "mpls-btm 0x%04x,0x%04x\n",yy_filter_rule.mpls_bottom.data, yy_filter_rule.mpls_bottom.mask);
        fprintf( out, "vlan-1 0x%04x,0x%04x\n",yy_filter_rule.vlan_1.data, yy_filter_rule.vlan_1.mask);
        fprintf( out, "vlan-2 0x%04x,0x%04x\n",yy_filter_rule.vlan_2.data, yy_filter_rule.vlan_2.mask);
        if ( k128Bit != yy_filter_rule.ip_address_type )
        {
            fprintf( out, "dst-ip 0x%04x,0x%04x\n",(uint32_t)yy_filter_rule.dst_ip.data[0], (uint32_t)yy_filter_rule.dst_ip.mask[0]);
            fprintf( out, "src-ip 0x%04x,0x%04x\n",(uint32_t) yy_filter_rule.src_ip.data[0], (uint32_t) yy_filter_rule.src_ip.mask[0]);
        }
        else 
        {
            fprintf( out, "dst-ip128 0x%016"PRIx64"%016"PRIx64",0x%016"PRIx64"%016"PRIx64" line_no: %d\n",yy_filter_rule.dst_ip.data[0],yy_filter_rule.dst_ip.data[1], yy_filter_rule.dst_ip.mask[0],yy_filter_rule.dst_ip.mask[1], bfslineno );
            fprintf( out, "src-ip 0x%016"PRIx64"%016"PRIx64",0x%016"PRIx64"%016"PRIx64" line_no: %d\n",yy_filter_rule.src_ip.data[0],yy_filter_rule.src_ip.data[1], yy_filter_rule.src_ip.mask[0],yy_filter_rule.src_ip.mask[1], bfslineno );
        }
        fprintf( out, "ip-prot 0x%04x,0x%04x\n",yy_filter_rule.ip_prot.data, yy_filter_rule.ip_prot.mask);
        fprintf( out, "src-port 0x%04x,0x%04x\n",yy_filter_rule.src_port.data, yy_filter_rule.src_port.mask);
        fprintf( out, "dst-port 0x%04x,0x%04x\n",yy_filter_rule.dst_port.data, yy_filter_rule.dst_port.mask);
        fprintf( out, "tcp-flags 0x%04x,0x%04x\n",yy_filter_rule.tcp_flags.data, yy_filter_rule.tcp_flags.mask);
		fprintf( out,"END of rule: on lineno:%d\n",bfslineno-1);
	#endif
		BEGIN(INITIAL);

		/* 
         *   Adding the end of file rules so that it accepts the last rule in a file if there 
         *   is no  new line 
		 */
		return T_RULE_DONE;
	}
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 1044 "bfs_filter_proto.l"
ECHO;
	YY_BREAK
#line 2474 "bfs_filter_proto.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(S_l2_proto):
case YY_STATE_EOF(S_mpls_top):
case YY_STATE_EOF(S_mpls_bottom):
case YY_STATE_EOF(S_vlan_1):
case YY_STATE_EOF(S_vlan_2):
case YY_STATE_EOF(S_dst_ip):
case YY_STATE_EOF(S_src_ip):
case YY_STATE_EOF(S_ip_prot):
case YY_STATE_EOF(S_src_port):
case YY_STATE_EOF(S_dst_port):
case YY_STATE_EOF(S_tcp_flags):
case YY_STATE_EOF(S_comment):
case YY_STATE_EOF(S_label_cnt):
case YY_STATE_EOF(S_iface):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed bfsin at a new source and called
			 * bfslex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = bfsin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( bfswrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * bfstext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of bfslex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					bfsrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			bfsrestart(bfsin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) bfsrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 353 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 353 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 352);

	return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					bfsrestart(bfsin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( bfswrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve bfstext */
	(yy_hold_char) = *++(yy_c_buf_p);

	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');
	if ( YY_CURRENT_BUFFER_LVALUE->yy_at_bol )
		   
    bfslineno++;
;

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void bfsrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        bfsensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            bfs_create_buffer(bfsin,YY_BUF_SIZE );
	}

	bfs_init_buffer(YY_CURRENT_BUFFER,input_file );
	bfs_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void bfs_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		bfspop_buffer_state();
	 *		bfspush_buffer_state(new_buffer);
     */
	bfsensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	bfs_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (bfswrap()) processing, but the only time this flag
	 * is looked at is after bfswrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void bfs_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	bfsin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE bfs_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) bfsalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in bfs_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) bfsalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in bfs_create_buffer()" );

	b->yy_is_our_buffer = 1;

	bfs_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with bfs_create_buffer()
 * 
 */
    void bfs_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		bfsfree((void *) b->yy_ch_buf  );

	bfsfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a bfsrestart() or at EOF.
 */
    static void bfs_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	bfs_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then bfs_init_buffer was _probably_
     * called from bfsrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void bfs_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		bfs_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void bfspush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	bfsensure_buffer_stack();

	/* This block is copied from bfs_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from bfs_switch_to_buffer. */
	bfs_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void bfspop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	bfs_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		bfs_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void bfsensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)bfsalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in bfsensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)bfsrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in bfsensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE bfs_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) bfsalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in bfs_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	bfs_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to bfslex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       bfs_scan_bytes() instead.
 */
YY_BUFFER_STATE bfs_scan_string (yyconst char * yystr )
{
    
	return bfs_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to bfslex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE bfs_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) bfsalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in bfs_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = bfs_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in bfs_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

    static void yy_push_state (int  new_state )
{
    	if ( (yy_start_stack_ptr) >= (yy_start_stack_depth) )
		{
		yy_size_t new_size;

		(yy_start_stack_depth) += YY_START_STACK_INCR;
		new_size = (yy_start_stack_depth) * sizeof( int );

		if ( ! (yy_start_stack) )
			(yy_start_stack) = (int *) bfsalloc(new_size  );

		else
			(yy_start_stack) = (int *) bfsrealloc((void *) (yy_start_stack),new_size  );

		if ( ! (yy_start_stack) )
			YY_FATAL_ERROR( "out of memory expanding start-condition stack" );
		}

	(yy_start_stack)[(yy_start_stack_ptr)++] = YY_START;

	BEGIN(new_state);
}

    static void yy_pop_state  (void)
{
    	if ( --(yy_start_stack_ptr) < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN((yy_start_stack)[(yy_start_stack_ptr)]);
}

    static int yy_top_state  (void)
{
    	return (yy_start_stack)[(yy_start_stack_ptr) - 1];
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up bfstext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		bfstext[bfsleng] = (yy_hold_char); \
		(yy_c_buf_p) = bfstext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		bfsleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int bfsget_lineno  (void)
{
        
    return bfslineno;
}

/** Get the input stream.
 * 
 */
FILE *bfsget_in  (void)
{
        return bfsin;
}

/** Get the output stream.
 * 
 */
FILE *bfsget_out  (void)
{
        return bfsout;
}

/** Get the length of the current token.
 * 
 */
int bfsget_leng  (void)
{
        return bfsleng;
}

/** Get the current token.
 * 
 */

char *bfsget_text  (void)
{
        return bfstext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void bfsset_lineno (int  line_number )
{
    
    bfslineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see bfs_switch_to_buffer
 */
void bfsset_in (FILE *  in_str )
{
        bfsin = in_str ;
}

void bfsset_out (FILE *  out_str )
{
        bfsout = out_str ;
}

int bfsget_debug  (void)
{
        return bfs_flex_debug;
}

void bfsset_debug (int  bdebug )
{
        bfs_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from bfslex_destroy(), so don't allocate here.
     */

    /* We do not touch bfslineno unless the option is enabled. */
    bfslineno =  1;
    
    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

    (yy_start_stack_ptr) = 0;
    (yy_start_stack_depth) = 0;
    (yy_start_stack) =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    bfsin = stdin;
    bfsout = stdout;
#else
    bfsin = (FILE *) 0;
    bfsout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * bfslex_init()
     */
    return 0;
}

/* bfslex_destroy is for both reentrant and non-reentrant scanners. */
int bfslex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		bfs_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		bfspop_buffer_state();
	}

	/* Destroy the stack itself. */
	bfsfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Destroy the start condition stack. */
        bfsfree((yy_start_stack)  );
        (yy_start_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * bfslex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *bfsalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *bfsrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void bfsfree (void * ptr )
{
	free( (char *) ptr );	/* see bfsrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 1044 "bfs_filter_proto.l"


/*			
int scanner_set_stdout(FILE *scanner_out) {
		out = scanner_out;
		return 0;
}
*/

int get_scanner_topstate_bfs(void) {
	return yy_top_state();
}


